@@ -21,7 +21,7 @@
  */
 
 static const char rcsid [] =
-"$Id: io-v4l.c,v 1.38 2008/02/24 14:17:28 mschimek Exp $";
+"$Id: io-v4l.c,v 1.39 2013-07-02 04:04:04 mschimek Exp $";
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
@@ -42,6 +42,7 @@ static const char rcsid [] =
 #include <assert.h>
 #include <sys/time.h>		/* timeval */
 #include <sys/types.h>		/* fd_set, uid_t */
+#include <sys/stat.h>		/* S_ISCHR */
 #include <sys/ioctl.h>		/* for (_)videodev.h */
 #include <pthread.h>
 
@@ -539,7 +540,6 @@ open_video_dev(vbi_capture_v4l *v, struct stat *p_vbi_stat, vbi_bool do_dev_scan
 		"/dev/v4l/video3",
 	};
 	struct dirent *dirent;
-	struct dirent *pdirent;
 	DIR *dir;
 	int video_fd;
 	unsigned int i;
@@ -568,8 +568,7 @@ open_video_dev(vbi_capture_v4l *v, struct stat *p_vbi_stat, vbi_bool do_dev_scan
 			goto done;
 		}
 
-		while (0 == readdir_r (dir, dirent, &pdirent)
-		       && pdirent == dirent) {
+		while (dirent == readdir (dir)) {
 			char name[256];
 
 			snprintf (name, sizeof(name),
