@@ -1,23 +1,26 @@
 /*
- *  libzvbi - Device interfaces
+ *  libzvbi -- VBI device interfaces
  *
  *  Copyright (C) 2002 Michael H. Schimek
+ *  Copyright (C) 2003, 2004 Tom Zoerner
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Library General Public
+ *  License as published by the Free Software Foundation; either
+ *  version 2 of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Library General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU Library General Public
+ *  License along with this library; if not, write to the 
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
+ *  Boston, MA  02110-1301  USA.
  */
 
-/* $Id: io.h,v 1.23 2007/11/27 18:31:06 mschimek Exp $ */
+/* $Id: io.h,v 1.25 2008-02-24 14:16:28 mschimek Exp $ */
 
 #ifndef IO_H
 #define IO_H
@@ -67,7 +70,7 @@ typedef enum {
         * not the actual device.  In this case it can only be
         * used for select(2) and not for ioctl(2)
         */
-        VBI_FD_IS_DEVICE   = 1<<2,
+        VBI_FD_IS_DEVICE   = 1<<2
 } VBI_CAPTURE_FD_FLAGS;
 
 /**
@@ -98,9 +101,18 @@ extern vbi_capture *	vbi_capture_bktr_new (const char *	dev_name,
 					      char **		errstr,
 					      vbi_bool		trace);
 extern int		vbi_capture_dvb_filter(vbi_capture *cap, int pid);
-extern vbi_capture*	vbi_capture_dvb_new(char *dev, int scanning,
-				        unsigned int *services, int strict,
-				        char **errstr, vbi_bool trace);
+
+/* This function is deprecated. Use vbi_capture_dvb_new2() instead.
+   See io-dvb.c or the Doxygen documentation for details. */
+extern vbi_capture *
+vbi_capture_dvb_new		(char *			dev,
+				 int			scanning,
+				 unsigned int *		services,
+				 int			strict,
+				 char **		errstr,
+				 vbi_bool		trace)
+  _vbi_deprecated;
+
 extern int64_t
 vbi_capture_dvb_last_pts	(const vbi_capture *	cap);
 extern vbi_capture *
