@@ -19,7 +19,7 @@
  *  Boston, MA  02110-1301  USA.
  */
 
-/* $Id: packet.c,v 1.29 2008/02/19 00:35:20 mschimek Exp $ */
+/* $Id: packet.c,v 1.34 2014-02-18 16:56:03 mschimek Exp $ */
 
 #include "site_def.h"
 
@@ -42,6 +42,7 @@
 #include "vps.h"
 #include "vbi.h"
 #include "cache-priv.h"
+#include "packet-830.h"
 
 #ifndef FPC
 #  define FPC 0
@@ -1074,14 +1075,6 @@ vbi_convert_page(vbi_decoder *vbi, cache_page *vtp,
 	}
 }
 
-typedef enum {
-	CNI_NONE,
-	CNI_VPS,	/* VPS format */
-	CNI_8301,	/* Teletext packet 8/30 format 1 */
-	CNI_8302,	/* Teletext packet 8/30 format 2 */
-	CNI_X26		/* Teletext packet X/26 local enhancement */
-} vbi_cni_type;
-
 static unsigned int
 station_lookup(vbi_cni_type type, int cni, const char **country, const char **name)
 {
@@ -1091,7 +1084,7 @@ station_lookup(vbi_cni_type type, int cni, const char **country, const char **na
 		return 0;
 
 	switch (type) {
-	case CNI_8301:
+	case VBI_CNI_TYPE_8301:
 		for (p = vbi_cni_table; p->name; p++)
 			if (p->cni1 == cni) {
 				*country = p->country;
@@ -1100,7 +1093,7 @@ station_lookup(vbi_cni_type type, int cni, const char **country, const char **na
 			}
 		break;
 
-	case CNI_8302:
+	case VBI_CNI_TYPE_8302:
 		for (p = vbi_cni_table; p->name; p++)
 			if (p->cni2 == cni) {
 				*country = p->country;
@@ -1112,7 +1105,7 @@ station_lookup(vbi_cni_type type, int cni, const char **country, const char **na
 
 		/* fall through */
 
-	case CNI_VPS:
+	case VBI_CNI_TYPE_VPS:
 		/* if (cni == 0x0DC3) in decoder
 			cni = mark ? 0x0DC2 : 0x0DC1; */
 
@@ -1124,7 +1117,7 @@ station_lookup(vbi_cni_type type, int cni, const char **country, const char **na
 			}
 		break;
 
-	case CNI_X26:
+	case VBI_CNI_TYPE_PDC_B:
 		for (p = vbi_cni_table; p->name; p++)
 			if (p->cni3 == cni) {
 				*country = p->country;
@@ -1166,7 +1159,8 @@ unknown_cni(vbi_decoder *vbi, const char *dl, int cni)
  * 
  * Decode a VPS datagram (13 bytes) according to
  * ETS 300 231 and update decoder state. This may
- * send a @a VBI_EVENT_NETWORK or @a VBI_EVENT_NETWORK_ID.
+ * send a @a VBI_EVENT_NETWORK, @a VBI_EVENT_NETWORK_ID
+ * or @a VBI_EVENT_PROG_ID.
  */
 void
 vbi_decode_vps(vbi_decoder *vbi, uint8_t *buf)
@@ -1180,10 +1174,14 @@ vbi_decode_vps(vbi_decoder *vbi, uint8_t *buf)
 	if (cni != (unsigned int) n->cni_vps) {
 		n->cni_vps = cni;
 		n->cycle = 1;
+
+		CLEAR (vbi->vps_pid);
+		/* May fail, leaving vbi->vps_pid unmodified. */
+		vbi_decode_vps_pdc (&vbi->vps_pid, buf);
 	} else if (n->cycle == 1) {
 		unsigned int id;
 
-		id = station_lookup(CNI_VPS, cni, &country, &name);
+		id = station_lookup(VBI_CNI_TYPE_VPS, cni, &country, &name);
 
 		if (0 == id) {
 			n->name[0] = 0;
@@ -1207,6 +1205,34 @@ vbi_decode_vps(vbi_decoder *vbi, uint8_t *buf)
 		vbi_send_event(vbi, &vbi->network);
 
 		n->cycle = 2;
+
+		if (vbi->event_mask & VBI_EVENT_PROG_ID) {
+			vbi_program_id pid;
+			vbi_event e;
+
+			CLEAR (pid);
+			if (!vbi_decode_vps_pdc (&pid, buf))
+				return;
+
+			/* VPS has no error protection so we send an
+			   event only after we receive a PID twice. */
+			if (0 != memcmp (&pid, &vbi->vps_pid,
+					 sizeof (pid))) {
+				vbi->vps_pid = pid;
+				return;
+			}
+
+			/* We also send an event if the PID did not
+			   change so the app can see if the signal is
+			   still present. */
+
+			CLEAR (e);
+
+			e.type = VBI_EVENT_PROG_ID;
+			e.ev.prog_id = &pid;
+
+			vbi_send_event (vbi, &e);
+		}
 	}
 }
 
@@ -1239,7 +1265,7 @@ parse_bsd(vbi_decoder *vbi, uint8_t *raw, int packet, int designation)
 			} else if (n->cycle == 1) {
 				unsigned int id;
 
-				id = station_lookup(CNI_8301, cni, &country, &name);
+				id = station_lookup(VBI_CNI_TYPE_8301, cni, &country, &name);
 
 				if (!id) {
 					n->name[0] = 0;
@@ -1267,7 +1293,7 @@ parse_bsd(vbi_decoder *vbi, uint8_t *raw, int packet, int designation)
 			}
 #if 0
 			if (1) { /* country and network identifier */
-				if (station_lookup(CNI_8301, cni, &country, &name))
+				if (station_lookup(VBI_CNI_TYPE_8301, cni, &country, &name))
 					printf("... country: %s\n... station: %s\n", country, name);
 				else
 					printf("... unknown CNI %04x\n", cni);
@@ -1329,7 +1355,7 @@ parse_bsd(vbi_decoder *vbi, uint8_t *raw, int packet, int designation)
 			} else if (n->cycle == 1) {
 				unsigned int id;
 
-				id = station_lookup(CNI_8302, cni, &country, &name);
+				id = station_lookup(VBI_CNI_TYPE_8302, cni, &country, &name);
 
 				if (!id) {
 					n->name[0] = 0;
@@ -1360,7 +1386,7 @@ parse_bsd(vbi_decoder *vbi, uint8_t *raw, int packet, int designation)
 			if (1) { /* country and network identifier */
 				const char *country, *name;
 
-				if (station_lookup(CNI_8302, cni, &country, &name))
+				if (station_lookup(VBI_CNI_TYPE_8302, cni, &country, &name))
 					printf("... country: %s\n... station: %s\n", country, name);
 				else
 					printf("... unknown CNI %04x\n", cni);
@@ -1419,8 +1445,8 @@ parse_bsd(vbi_decoder *vbi, uint8_t *raw, int packet, int designation)
 }
 
 static int
-same_header(int cur_pgno, uint8_t *cur,
-	    int ref_pgno, uint8_t *ref,
+same_header(int cur_pgno, const uint8_t *cur,
+	    int ref_pgno, const uint8_t *ref,
 	    int *page_num_offsetp)
 {
 	uint8_t buf[3];
@@ -1477,7 +1503,7 @@ same_header(int cur_pgno, uint8_t *cur,
 }
 
 static inline vbi_bool
-same_clock(uint8_t *cur, uint8_t *ref)
+same_clock(const uint8_t *cur, const uint8_t *ref)
 {
 	int i;
 
@@ -1644,6 +1670,92 @@ store_lop(vbi_decoder *vbi, const cache_page *vtp)
 	return TRUE;
 }
 
+static void
+lop_parity_check		(cache_page *		cvtp,
+				 struct raw_page *	rvtp)
+{
+	if (0 != cvtp->x26_designations) {
+		struct ttx_triplet *trip = cvtp->data.enh_lop.enh;
+		struct ttx_triplet *trip_end = trip + N_ELEMENTS (cvtp->data.enh_lop.enh);
+		unsigned int row = 0;
+
+		/* This is a little work-around for Teletext encoders
+		   which transmit X/26 fallback characters with even
+		   parity as noted in EN 300 706 Table 25. The page
+		   formatting code can detect parity errors and pick a
+		   replacement character, however the parity check
+		   below attempts to correct errors when a page is
+		   retransmitted, and requires odd parity on all
+		   characters. */
+
+		for (; trip < trip_end; ++trip) {
+			if (trip->address < 40) {
+				switch (trip->mode) {
+				case 0x01: /* G1 block mosaic character */
+				case 0x02: /* G3 smooth mosaic or line drawing character */
+				case 0x0B: /* G3 smooth mosaic or line drawing character */
+				case 0x08: /* modified G0 and G2 character set designation */
+				case 0x09: /* G0 character */
+				case 0x0D: /* drcs character invocation */
+				case 0x0F: /* G2 character */
+				case 0x10 ... 0x1F: /* characters including diacritical marks */
+				{
+					unsigned int column = trip->address;
+					unsigned int c = rvtp->lop_raw[row][column];
+					rvtp->lop_raw[row][column] = vbi_par8 (c);
+					break;
+				}
+				default:
+					break;
+				}
+			} else if (trip->address > 63) {
+				/* Missed triplet or uncorrectable transmission error. */
+				break;
+			} else {
+				switch (trip->mode) {
+				case 0x01: /* full row colour */
+				case 0x04: /* set active position */
+					row = trip->address - 40;
+					if (0 == row)
+						row = 24;
+					break;
+				case 0x07: /* address display row 0 */
+					row = 0;
+					break;
+				default:
+					break;
+				}
+			}
+		}
+	}
+
+	/* Level 1 parity check. */
+
+	{
+		unsigned int packet;
+
+		for (packet = 1; packet <= 25; ++packet) {
+			unsigned int i;
+			int n;
+
+			if (0 == (rvtp->lop_packets & (1 << packet)))
+				continue;
+
+			n = 0;
+			for (i = 0; i < 40; ++i)
+				n |= vbi_unpar8 (rvtp->lop_raw[packet][i]);
+			if (n >= 0) {
+				/* Parity is good, replace cached row. We
+				   could replace individual characters, but
+				   a single parity bit isn't very reliable. */
+				memcpy (cvtp->data.lop.raw[packet],
+					rvtp->lop_raw[packet], 40);
+				cvtp->lop_packets |= 1 << packet;
+			}
+		}
+	}
+}
+
 #define TTX_EVENTS (VBI_EVENT_TTX_PAGE)
 #define BSDATA_EVENTS (VBI_EVENT_NETWORK | VBI_EVENT_NETWORK_ID)
 
@@ -1734,6 +1846,36 @@ if(0)
 	return TRUE;
 }
 
+struct bit_stream {
+	int *			triplet;
+	unsigned int		buffer;
+	unsigned int		left;
+};
+
+static unsigned int
+get_bits			(struct	bit_stream *	bs,
+				 unsigned int		count)
+{
+	unsigned int r;
+	int n;
+
+	r = bs->buffer;
+	n = count - bs->left;
+
+	if (n > 0) {
+		bs->buffer = *(bs->triplet)++;
+		r |= bs->buffer << bs->left;
+		bs->left = 18 - n;
+	} else {
+		n = count;
+		bs->left -= count;
+	}
+
+	bs->buffer >>= n;
+
+	return r & ((1UL << count) - 1);
+}
+
 /*
  *  Teletext packets 28 and 29, Level 2.5/3.5 enhancement
  */
@@ -1741,31 +1883,12 @@ static vbi_bool
 parse_28_29(vbi_decoder *vbi, uint8_t *p,
 	    cache_page *cvtp, int mag8, int packet)
 {
-	int designation, function;
-	int triplets[13], *triplet = triplets, buf = 0, left = 0;
+	int designation, function, coding;
+	int triplets[13];
+	struct bit_stream bs;
 	struct ttx_extension *ext;
 	int i, j, err = 0;
 
-	/* XXX nested function not portable, to be removed */
-	int
-	bits(int count)
-	{
-		int r, n;
-
-		r = buf;
-
-		if ((n = count - left) > 0) {
-			r |= (buf = *triplet++) << left;
-			left = 18;
-		} else
-			n = count;
-
-		buf >>= n;
-		left -= n;
-
-		return r & ((1UL << count) - 1);
-	}
-
 	if ((designation = vbi_unham8 (*p)) < 0)
 		return FALSE;
 
@@ -1774,7 +1897,11 @@ parse_28_29(vbi_decoder *vbi, uint8_t *p,
 			mag8, packet, designation, cvtp->pgno);
 
 	for (p++, i = 0; i < 13; p += 3, i++)
-		err |= triplet[i] = vbi_unham24p (p);
+		err |= triplets[i] = vbi_unham24p (p);
+
+	bs.triplet = triplets;
+	bs.buffer = 0;
+	bs.left = 0;
 
 	switch (designation) {
 	case 0: /* X/28/0, M/29/0 Level 2.5 */
@@ -1782,8 +1909,8 @@ parse_28_29(vbi_decoder *vbi, uint8_t *p,
 		if (err < 0)
 			return FALSE;
 
-		function = bits(4);
-		bits(3); /* page coding ignored */
+		function = get_bits (&bs, 4);
+		coding = get_bits (&bs, 3); /* page coding ignored */
 
 //		printf("... function %d\n", function);
 
@@ -1819,24 +1946,24 @@ parse_28_29(vbi_decoder *vbi, uint8_t *p,
 		}
 
 		if (designation == 4 && (ext->designations & (1 << 0)))
-			bits(14 + 2 + 1 + 4);
+			get_bits (&bs, 14 + 2 + 1 + 4);
 		else {
 			vbi_bool left_panel;
 			vbi_bool right_panel;
 			unsigned int left_columns;
 
-			ext->charset_code[0] = bits(7);
-			ext->charset_code[1] = bits(7);
+			ext->charset_code[0] = get_bits (&bs, 7);
+			ext->charset_code[1] = get_bits (&bs, 7);
 
-			left_panel = bits (1);
-			right_panel = bits (1);
+			left_panel = get_bits (&bs, 1);
+			right_panel = get_bits (&bs, 1);
 
 			/* 0 - panels required at Level 3.5 only,
 			   1 - at 2.5 and 3.5
 			   ignored. */
-			bits (1);
+			get_bits (&bs, 1);
 
-			left_columns = bits (4);
+			left_columns = get_bits (&bs, 4);
 
 			if (left_panel && 0 == left_columns)
 				left_columns = 16;
@@ -1850,7 +1977,7 @@ parse_28_29(vbi_decoder *vbi, uint8_t *p,
 		j = (designation == 4) ? 16 : 32;
 
 		for (i = j - 16; i < j; i++) {
-			vbi_rgba col = bits(12);
+			vbi_rgba col = get_bits (&bs, 12);
 
 			if (i == 8) /* transparent */
 				continue;
@@ -1863,14 +1990,14 @@ parse_28_29(vbi_decoder *vbi, uint8_t *p,
 		}
 
 		if (designation == 4 && (ext->designations & (1 << 0)))
-			bits(10 + 1 + 3);
+			get_bits (&bs, 10 + 1 + 3);
 		else {
-			ext->def_screen_color = bits(5);
-			ext->def_row_color = bits(5);
+			ext->def_screen_color = get_bits (&bs, 5);
+			ext->def_row_color = get_bits (&bs, 5);
 
-			ext->fallback.black_bg_substitution = bits(1);
+			ext->fallback.black_bg_substitution = get_bits (&bs, 1);
 
-			i = bits(3); /* color table remapping */
+			i = get_bits (&bs, 3); /* color table remapping */
 
 			ext->foreground_clut = "\00\00\00\10\10\20\20\20"[i];
 			ext->background_clut = "\00\10\20\10\20\10\20\30"[i];
@@ -1914,13 +2041,15 @@ parse_28_29(vbi_decoder *vbi, uint8_t *p,
 			/* XXX TODO - lop? */
 		}
 
-		triplet++;
+		/* 9.4.4: "Compatibility, not for Level 2.5/3.5 decoders."
+		   No more details, so we ignore this triplet. */
+		++bs.triplet;
 
 		for (i = 0; i < 8; i++)
-			ext->drcs_clut[i + 2] = vbi_rev8 (bits(5)) >> 3;
+			ext->drcs_clut[i + 2] = vbi_rev8 (get_bits (&bs, 5)) >> 3;
 
 		for (i = 0; i < 32; i++)
-			ext->drcs_clut[i + 10] = vbi_rev8 (bits(5)) >> 3;
+			ext->drcs_clut[i + 10] = vbi_rev8 (get_bits (&bs, 5)) >> 3;
 
 		ext->designations |= 1 << 1;
 
@@ -1936,11 +2065,11 @@ parse_28_29(vbi_decoder *vbi, uint8_t *p,
 		if (err < 0)
 			return FALSE;
 
-		function = bits(4);
-		bits(3); /* page coding ignored */
+		function = get_bits (&bs, 4);
+		coding = get_bits (&bs, 3); /* page coding ignored */
 
 		if (function != PAGE_FUNCTION_GDRCS
-		    || function != PAGE_FUNCTION_DRCS)
+		    && function != PAGE_FUNCTION_DRCS)
 			return FALSE;
 
 		if (cvtp->function == PAGE_FUNCTION_UNKNOWN) {
@@ -1957,10 +2086,10 @@ parse_28_29(vbi_decoder *vbi, uint8_t *p,
 			return 0;
 		}
 
-		bits(11);
+		get_bits (&bs, 11);
 
 		for (i = 0; i < 48; i++)
-			cvtp->data.drcs.mode[i] = bits(4);
+			cvtp->data.drcs.mode[i] = get_bits (&bs, 4);
 
 	default: /* ? */
 		break;
@@ -1973,14 +2102,17 @@ parse_28_29(vbi_decoder *vbi, uint8_t *p,
  *  Teletext packet 8/30, broadcast service data
  */
 static inline vbi_bool
-parse_8_30(vbi_decoder *vbi, uint8_t *p, int packet)
+parse_8_30(vbi_decoder *vbi, uint8_t *buffer, int packet)
 {
+	uint8_t *p;
 	int designation;
 
+	p = buffer + 2;
+
 	if ((designation = vbi_unham8 (*p)) < 0)
 		return FALSE;
 
-//	printf("Packet 8/30/%d\n", designation);
+	// printf("Packet 8/30/%d\n", designation);
 
 	if (designation > 4)
 		return TRUE; /* ignored */
@@ -1995,8 +2127,50 @@ parse_8_30(vbi_decoder *vbi, uint8_t *p, int packet)
 		}
 	}
 
-	if (vbi->event_mask & BSDATA_EVENTS)
-		return parse_bsd(vbi, p, packet, designation);
+	if (vbi->event_mask & BSDATA_EVENTS) {
+		if (!parse_bsd(vbi, p, packet, designation))
+			return FALSE;
+	}
+
+	if (designation < 2) {
+		/* 8/30 format 1 */
+
+		if (vbi->event_mask & VBI_EVENT_LOCAL_TIME) {
+			vbi_local_time lt;
+			vbi_event e;
+
+			CLEAR (e);
+
+			if (!vbi_decode_teletext_8301_local_time
+			    (&lt.time, &lt.seconds_east, buffer))
+				return FALSE;
+
+			lt.seconds_east_valid = TRUE;
+			lt.dst_state = VBI_DST_INCLUDED;
+
+			e.type = VBI_EVENT_LOCAL_TIME;
+			e.ev.local_time	= &lt;
+
+			vbi_send_event (vbi, &e);
+		}
+	} else {
+		/* 8/30 format 2 */
+
+		if (vbi->event_mask & VBI_EVENT_PROG_ID) {
+			vbi_program_id pid;
+			vbi_event e;
+			
+			if (!vbi_decode_teletext_8302_pdc (&pid, buffer))
+				return FALSE;
+
+			CLEAR (e);
+
+			e.type = VBI_EVENT_PROG_ID;
+			e.ev.prog_id = &pid;
+
+			vbi_send_event (vbi, &e);
+		}
+	}
 
 	return TRUE;
 }
@@ -2010,15 +2184,18 @@ parse_8_30(vbi_decoder *vbi, uint8_t *p, int packet)
  * state accordingly. This function may send events.
  * 
  * Return value:
- * FALSE if the packet contained incorrectable errors. 
+ * FALSE if the packet contained uncorrectable errors. 
  */
 vbi_bool
-vbi_decode_teletext(vbi_decoder *vbi, uint8_t *p)
+vbi_decode_teletext(vbi_decoder *vbi, uint8_t *buffer)
 {
 	cache_page *cvtp;
 	struct raw_page *rvtp;
 	int pmag, mag0, mag8, packet;
 	struct ttx_magazine *mag;
+	uint8_t *p;
+
+	p = buffer;
 
 	if ((pmag = vbi_unham16p (p)) < 0)
 		return FALSE;
@@ -2085,6 +2262,7 @@ vbi_decode_teletext(vbi_decoder *vbi, uint8_t *p)
 				break;
 
 			case PAGE_FUNCTION_LOP:
+				lop_parity_check(vtp, curr);
 				if (!store_lop(vbi, vtp))
 					return FALSE;
 				break;
@@ -2336,6 +2514,7 @@ vbi_decode_teletext(vbi_decoder *vbi, uint8_t *p)
 		}
 //XXX?
 		cvtp->data.ext_lop.ext.designations = 0;
+		rvtp->lop_packets = 0;
 		rvtp->num_triplets = 0;
 
 		return TRUE;
@@ -2394,6 +2573,13 @@ vbi_decode_teletext(vbi_decoder *vbi, uint8_t *p)
 			break;
 
 		case PAGE_FUNCTION_LOP:
+			/* Parity check postponed until we received the X/26
+			   enhancement packets pertaining to this page.
+			   See lop_parity_check(). */
+			memcpy(rvtp->lop_raw[packet], p, 40);
+			rvtp->lop_packets |= 1 << packet;
+			return TRUE;
+
 		case PAGE_FUNCTION_EACEM_TRIGGER:
 			for (n = i = 0; i < 40; i++)
 				n |= vbi_unpar8 (p[i]);
@@ -2495,7 +2681,7 @@ vbi_decode_teletext(vbi_decoder *vbi, uint8_t *p)
 		 */
 		switch (/* Channel */ pmag & 15) {
 		case 0: /* Packet 8/30 (ETS 300 706) */
-			if (!parse_8_30(vbi, p, packet))
+			if (!parse_8_30(vbi, buffer, packet))
 				return FALSE;
 			break;
 
