@@ -1,22 +1,25 @@
 /*
- *  Basic I/O between VBI proxy client & server
+ *  libvbi -- Basic I/O between VBI proxy client & server
  *
- *  Copyright (C) 2003,2004 Tom Zoerner
+ *  Copyright (C) 2003, 2004 Tom Zoerner
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Library General Public
+ *  License as published by the Free Software Foundation; either
+ *  version 2 of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Library General Public License for more details.
  *
+ *  You should have received a copy of the GNU Library General Public
+ *  License along with this library; if not, write to the 
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
+ *  Boston, MA  02110-1301  USA.
+ */
+
+/*
  *  Description:
  *
  *    This module contains a collection of functions for lower-level
@@ -28,10 +31,15 @@
  *
  *    Both UNIX domain and IPv4 and IPv6 sockets are implemented, but
  *    the latter ones are currently not officially supported.
+ */
+ 
+/*
+ *  $Id: proxy-msg.c,v 1.21 2013-08-28 14:45:06 mschimek Exp $
  *
- *  $Id: proxy-msg.c,v 1.19 2007/11/27 18:31:07 mschimek Exp $
+ *  $Log: not supported by cvs2svn $
+ *  Revision 1.20  2008/02/19 00:35:21  mschimek
+ *  *** empty log message ***
  *
- *  $Log: proxy-msg.c,v $
  *  Revision 1.19  2007/11/27 18:31:07  mschimek
  *  Updated the FSF address in the copyright notice.
  *
@@ -132,13 +140,20 @@
 #include "misc.h"
 #include "proxy-msg.h"
 
+#ifdef ENABLE_V4L2
+#include <asm/types.h>		/* __u8 and friends for videodev2k.h */
+#ifndef HAVE_S64_U64
+#  include <inttypes.h>
+  /* Linux 2.6.x asm/types.h defines __s64 and __u64 only
+     if __GNUC__ is defined. */
+typedef int64_t __s64;
+typedef uint64_t __u64;
+#endif
+#include "videodev2k.h"
+#endif
 #ifdef ENABLE_V4L
 #include "videodev.h"
 #endif
-#ifdef ENABLE_V4L2
-#include <asm/types.h>
-#include "videodev2k.h"
-#endif
 
 #define dprintf1(fmt, arg...)    do {if (proxy_msg_trace >= 1) fprintf(stderr, "proxy_msg: " fmt, ## arg);} while(0)
 #define dprintf2(fmt, arg...)    do {if (proxy_msg_trace >= 2) fprintf(stderr, "proxy_msg: " fmt, ## arg);} while(0)
@@ -1024,7 +1039,6 @@ int vbi_proxy_msg_accept_connection( int listen_fd )
 static char * vbi_proxy_msg_resolve_symlinks( const char * p_dev_name )
 {
    struct stat stbuf;
-   char   link_name[MAXPATHLEN + 1];
    char * p_path;
    char * p_tmp;
    char * p_tmp2;
@@ -1039,6 +1053,7 @@ static char * vbi_proxy_msg_resolve_symlinks( const char * p_dev_name )
       res = lstat(p_path, &stbuf);
       if ((res == 0) && S_ISLNK(stbuf.st_mode))
       {
+         char link_name[stbuf.st_size + 1];
          name_len = readlink(p_path, link_name, sizeof(link_name));
          if ((name_len > 0) && (name_len < (int) sizeof(link_name)))
          {
@@ -1315,7 +1330,7 @@ vbi_bool vbi_proxy_msg_finish_connect( int sock_fd, char ** ppErrorText )
 ** Query size and character of an ioctl request for v4l1 drivers
 */
 static int
-vbi_proxy_msg_v4l_ioctl( int request, void * p_arg, vbi_bool * req_perm )
+vbi_proxy_msg_v4l_ioctl( unsigned int request, void * p_arg, vbi_bool * req_perm )
 {
    p_arg = p_arg;
 
@@ -1359,7 +1374,7 @@ vbi_proxy_msg_v4l_ioctl( int request, void * p_arg, vbi_bool * req_perm )
 ** Query size and character of an ioctl request for v4l2 drivers
 */
 static int
-vbi_proxy_msg_v4l2_ioctl( int request, void * p_arg, vbi_bool * req_perm )
+vbi_proxy_msg_v4l2_ioctl( unsigned int request, void * p_arg, vbi_bool * req_perm )
 {
    switch (request)
    {
