@@ -1,24 +1,25 @@
 /*
- *  libzvbi - Sliced vbi data object
+ *  libzvbi -- Sliced VBI data
  *
  *  Copyright (C) 2000, 2001 Michael H. Schimek
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Library General Public
+ *  License as published by the Free Software Foundation; either
+ *  version 2 of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Library General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU Library General Public
+ *  License along with this library; if not, write to the 
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
+ *  Boston, MA  02110-1301  USA.
  */
 
-/* $Id: sliced.h,v 1.9 2007/11/27 17:41:37 mschimek Exp $ */
+/* $Id: sliced.h,v 1.11 2008-02-24 14:17:02 mschimek Exp $ */
 
 #ifndef SLICED_H
 #define SLICED_H
@@ -344,10 +345,10 @@ typedef struct {
  */
 extern const char *
 vbi_sliced_name			(vbi_service_set	service)
-  _vbi_attribute ((const));
+  _vbi_const;
 extern unsigned int
 vbi_sliced_payload_bits		(vbi_service_set	service)
-  _vbi_attribute ((const));
+  _vbi_const;
 /** @} */
 
 /* Private */
