@@ -1,24 +1,25 @@
 /*
- *  libzvbi - Extended Data Service demultiplexer
+ *  libzvbi -- Extended Data Service demultiplexer
  *
  *  Copyright (C) 2000-2005 Michael H. Schimek
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Library General Public
+ *  License as published by the Free Software Foundation; either
+ *  version 2 of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Library General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU Library General Public
+ *  License along with this library; if not, write to the 
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
+ *  Boston, MA  02110-1301  USA.
  */
 
-/* $Id: xds_demux.c,v 1.10 2007/11/27 18:31:07 mschimek Exp $ */
+/* $Id: xds_demux.c,v 1.11 2008-02-19 00:35:23 mschimek Exp $ */
 
 #include "site_def.h"
 
@@ -818,14 +819,17 @@ void
 vbi_xds_demux_reset		(vbi_xds_demux *	xd)
 {
 	unsigned int n;
-	unsigned int i;
+	unsigned int i, j;
 
 	assert (NULL != xd);
 
-	n = N_ELEMENTS (xd->subpacket) * N_ELEMENTS (xd->subpacket[0]);
+	n = N_ELEMENTS (xd->subpacket[0]);
 
-	for (i = 0; i < n; ++i)
-		xd->subpacket[0][i].count = 0;
+	for (i = 0; i < N_ELEMENTS (xd->subpacket); ++i) {
+		for (j = 0; j < N_ELEMENTS (xd->subpacket[0]); ++j) {
+			xd->subpacket[i][j].count = 0;
+		}
+	}
 
 	xd->curr_sp = NULL;
 }
