@@ -16,13 +16,16 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+ *  MA 02110-1301, USA.
  */
 
-/* $Id: decode.c,v 1.30 2007/11/27 18:26:48 mschimek Exp $ */
+/* $Id: decode.c,v 1.36 2009-12-14 23:43:52 mschimek Exp $ */
 
 /* For libzvbi version 0.2.x / 0.3.x. */
 
+#undef NDEBUG
+
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
@@ -43,14 +46,16 @@
 #include "src/version.h"
 #if 2 == VBI_VERSION_MINOR
 #  include "src/bcd.h"
+#  include "src/cc608_decoder.h"
 #  include "src/conv.h"
-#  include "src/pfc_demux.h"
 #  include "src/dvb_demux.h"
-#  include "src/idl_demux.h"
-#  include "src/xds_demux.h"
-#  include "src/vps.h"
 #  include "src/hamm.h"
+#  include "src/idl_demux.h"
 #  include "src/lang.h"
+#  include "src/packet-830.h"
+#  include "src/pfc_demux.h"
+#  include "src/vps.h"
+#  include "src/xds_demux.h"
 #elif 3 == VBI_VERSION_MINOR
 #  include "src/zvbi.h"
 #  include "src/misc.h"		/* _vbi_to_ascii() */
@@ -106,160 +111,6 @@ _vbi_pfc_block_dump		(const vbi_pfc_block *	pb,
 				 FILE *			fp,
 				 vbi_bool		binary);
 
-static void
-put_cc_char			(unsigned int		c1,
-				 unsigned int		c2)
-{
-	uint16_t ucs2_str[1];
-
-	/* All caption characters are representable
-	   in UTF-8, but not necessarily in ASCII. */
-	ucs2_str[0] = vbi_caption_unicode ((c1 * 256 + c2) & 0x777F,
-					    /* to_upper */ FALSE);
-
-	vbi_fputs_iconv_ucs2 (stdout,
-			       vbi_locale_codeset (),
-			       ucs2_str, 1,
-			       /* repl_char */ '?');
-}
-
-static void
-caption_command			(unsigned int		line,
-				 unsigned int		c1,
-				 unsigned int		c2)
-{
-	unsigned int ch;
-	unsigned int a7;
-	unsigned int f;
-	unsigned int b7;
-	unsigned int u;
-
-	printf ("CC line=%3u cmd 0x%02x 0x%02x ", line, c1, c2);
-
-	if (0 == c1) {
-		printf ("null\n");
-		return;
-	} else if (c2 < 0x20) {
-		printf ("invalid\n");
-		return;
-	}
-
-	/* Some common bit groups. */
-
-	ch = (c1 >> 3) & 1; /* channel */
-	a7 = c1 & 7;
-	f = c1 & 1; /* field */
-	b7 = (c2 >> 1) & 7;
-	u = c2 & 1; /* underline */
-
-	if (c2 >= 0x40) {
-		static const int row [16] = {
-			/* 0 */ 10,			/* 0x1040 */
-			/* 1 */ -1,			/* unassigned */
-			/* 2 */ 0, 1, 2, 3,		/* 0x1140 ... 0x1260 */
-			/* 6 */ 11, 12, 13, 14,		/* 0x1340 ... 0x1460 */
-			/* 10 */ 4, 5, 6, 7, 8, 9	/* 0x1540 ... 0x1760 */
-		};
-		unsigned int rrrr;
-
-		/* Preamble Address Codes -- 001 crrr  1ri bbbu */
-  
-		rrrr = a7 * 2 + ((c2 >> 5) & 1);
-
-		if (c2 & 0x10)
-			printf ("PAC ch=%u row=%d column=%u u=%u\n",
-				ch, row[rrrr], b7 * 4, u);
-		else
-			printf ("PAC ch=%u row=%d color=%u u=%u\n",
-				ch, row[rrrr], b7, u);
-		return;
-	}
-
-	/* Control codes -- 001 caaa  01x bbbu */
-
-	switch (a7) {
-	case 0:
-		if (c2 < 0x30) {
-			const char *mnemo [16] = {
-				"BWO", "BWS", "BGO", "BGS",
-				"BBO", "BBS", "BCO", "BCS",
-				"BRO", "BRS", "BYO", "BYS",
-				"BMO", "BMS", "BAO", "BAS"
-			};
-
-			printf ("%s ch=%u\n", mnemo[c2 & 0xF], ch);
-			return;
-		}
-
-		break;
-
-	case 1:
-		if (c2 < 0x30) {
-			printf ("mid-row ch=%u color=%u u=%u\n", ch, b7, u);
-		} else {
-			printf ("special character ch=%u 0x%02x%02x='",
-				ch, c1, c2);
-			put_cc_char (c1, c2);
-			puts ("'");
-		}
-
-		return;
-
-	case 2: /* first group */
-	case 3: /* second group */
-		printf ("extended character ch=%u 0x%02x%02x='", ch, c1, c2);
-		put_cc_char (c1, c2);
-		puts ("'");
-		return;
-
-	case 4: /* f=0 */
-	case 5: /* f=1 */
-		if (c2 < 0x30) {
-			const char *mnemo [16] = {
-				"RCL", "BS",  "AOF", "AON",
-				"DER", "RU2", "RU3", "RU4",
-				"FON", "RDC", "TR",  "RTD",
-				"EDM", "CR",  "ENM", "EOC"
-			};
-
-			printf ("%s ch=%u f=%u\n", mnemo[c2 & 0xF], ch, f);
-			return;
-		}
-
-		break;
-
-	case 6:
-		printf ("reserved\n");
-		return;
-
-	case 7:
-		switch (c2) {
-		case 0x21 ... 0x23:
-			printf ("TO%u ch=%u\n", c2 - 0x20, ch);
-			return;
-
-		case 0x2D:
-			printf ("BT ch=%u\n", ch);
-			return;
-
-		case 0x2E:
-			printf ("FA ch=%u\n", ch);
-			return;
-
-		case 0x2F:
-			printf ("FAU ch=%u\n", ch);
-			return;
-
-		default:
-			break;
-		}
-
-		break;
-	}
-
-	printf ("unknown\n");
-}
-
 static vbi_bool
 xds_cb				(vbi_xds_demux *	xd,
 				 const vbi_xds_packet *	xp,
@@ -286,48 +137,9 @@ caption				(const uint8_t		buffer[2],
 	if (option_decode_caption
 	    && (21 == line || 284 == line /* NTSC */
 		|| 22 == line /* PAL? */)) {
-		int c1;
-		int c2;
 
-		c1 = vbi_unpar8 (buffer[0]);
-		c2 = vbi_unpar8 (buffer[1]);
-
-		if ((c1 | c2) < 0) {
-			printf ("Parity error in CC line=%u "
-				" %s0x%02x %s0x%02x.\n",
-				line,
-				(c1 < 0) ? ">" : "", buffer[0] & 0xFF,
-				(c2 < 0) ? ">" : "", buffer[1] & 0xFF);
-		} else if (c1 >= 0x20) {
-			char text[2];
-
-			printf ("CC line=%3u text 0x%02x 0x%02x '",
-				line, c1, c2);
-
-			/* All caption characters are representable
-			   in UTF-8, but not necessarily in ASCII. */
-			text[0] = c1;
-			text[1] = c2; /* may be zero */
-
-			/* Error ignored. */
-			vbi_fputs_iconv (stdout,
-					  /* dst_codeset */
-					  vbi_locale_codeset (),
-					  /* src_codeset */ "EIA-608",
-					  text, 2,
-					  /* repl_char */ '?');
-
-			puts ("'");
-		} else if (0 == c1 || c1 >= 0x10) {
-			caption_command (line, c1, c2);
-		} else if (option_decode_xds) {
-			printf ("CC line=%3u cmd 0x%02x 0x%02x ",
-				line, c1, c2);
-			if (0x0F == c1)
-				puts ("XDS packet end");
-			else
-				puts ("XDS packet start/continue");
-		}
+		printf ("CC line=%3u ", line);
+		_vbi_cc608_dump (stdout, buffer[0], buffer[1]);
 	}
 }
 
@@ -389,8 +201,6 @@ dump_bytes			(const uint8_t *	buffer,
 	puts ("<");
 }
 
-#if 3 == VBI_VERSION_MINOR /* XXX port me back */
-
 static void
 packet_8301			(const uint8_t		buffer[42],
 				 unsigned int		designation)
@@ -424,8 +234,10 @@ packet_8301			(const uint8_t		buffer[42],
 		tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday,
 		tm.tm_hour, tm.tm_min, tm.tm_sec);
 
+#if 3 == VBI_VERSION_MINOR
 	if (0 != cni)
 		dump_cni (VBI_CNI_TYPE_8301, cni);
+#endif
 }
 
 static void
@@ -456,12 +268,12 @@ packet_8302			(const uint8_t		buffer[42],
 
 	putchar ('\n');
 
+#if 3 == VBI_VERSION_MINOR
 	if (0 != pi.cni)
 		dump_cni (pi.cni_type, pi.cni);
+#endif
 }
 
-#endif /* 3 == VBI_VERSION_MINOR */
-
 static vbi_bool
 page_function_clear_cb		(vbi_pfc_demux *	dx,
 				 void *			user_data,
@@ -663,16 +475,12 @@ teletext			(const uint8_t		buffer[42],
 		}
 
 		if (designation >= 0 && designation <= 1) {
-#if 3 == VBI_VERSION_MINOR /* XXX port me back */
 			packet_8301 (buffer, designation);
-#endif
 			return;
 		}
 
 		if (designation >= 2 && designation <= 3) {
-#if 3 == VBI_VERSION_MINOR /* XXX port me back */
 			packet_8302 (buffer, designation);
-#endif
 			return;
 		}
 	}
@@ -704,7 +512,7 @@ vps				(const uint8_t		buffer[13],
 {
 	if (option_decode_vps) {
 		unsigned int cni;
-#if 3 == VBI_VERSION_MINOR
+#if 1
 		vbi_program_id pi;
 #endif
 		if (option_dump_bin) {
@@ -719,22 +527,20 @@ vps				(const uint8_t		buffer[13],
 			return;
 		}
 
-#if 3 == VBI_VERSION_MINOR
 		if (!vbi_decode_vps_pdc (&pi, buffer)) {
 			printf ("Error in VPS packet PDC data.\n");
 			return;
 		}
-		
+
 		printf ("VPS line=%3u ", line);
 
 		_vbi_program_id_dump (&pi, stdout);
 
 		putchar ('\n');
 
+#if 3 == VBI_VERSION_MINOR
 		if (0 != pi.cni)
 			dump_cni (pi.cni_type, pi.cni);
-#else
-		printf ("VPS line=%3u CNI=%x\n", line, cni);
 #endif
 	}
 
@@ -894,12 +700,10 @@ Input options:\n\
                        standard input\n\
 -P | --pes             Source is a DVB PES stream\n\
 -T | --ts pid          Source is a DVB TS stream\n\
-Decoding options:\n"
-#if 3 == VBI_VERSION_MINOR /* XXX port me back */
-"-1 | --8301            Teletext packet 8/30 format 1 (local time)\n\
--2 | --8302            Teletext packet 8/30 format 2 (PDC)\n"
-#endif
-"-c | --cc              Closed Caption\n\
+Decoding options:\n\
+-1 | --8301            Teletext packet 8/30 format 1 (local time)\n\
+-2 | --8302            Teletext packet 8/30 format 2 (PDC)\n\
+-c | --cc              Closed Caption\n\
 -j | --idl             Any Teletext IDL packets (M/30, M/31)\n\
 -t | --ttx             Decode any Teletext packet\n\
 -v | --vps             Video Programming System (PDC)\n"
